#include <stdio.h>
#include <gudev/gudev.h>

#include "config.h"
#include "fd-dmi.h"

static gint
fd_udev_get_dmi_chassis_type(void)
{
    gint type = 0;
    GList *devices;
    GUdevClient *client;

    client = g_udev_client_new (NULL);
    devices = g_udev_client_query_by_subsystem (client, "dmi");
    if (devices == NULL)
        type = -1;

    while (devices != NULL){
            gint attr = g_udev_device_get_sysfs_attr_as_int(devices->data,"chassis_type");
            if(attr > 0){
                    type = (int)attr;
                    break;
            }
            devices = devices->next;
    }
    g_object_unref (client);
    g_list_foreach (devices, (GFunc)g_object_unref, NULL);
    g_list_free (devices);
    return type;
}

FdChassisType
fd_dmi_get_chassis_type (void)
{
    FdChassisType type;
    gint chassis_type = fd_udev_get_dmi_chassis_type ();
    if (chassis_type == -1)
    {
        gchar *file_path = "/etc/desktop/dmi";
        if (!g_file_test (file_path, G_FILE_TEST_EXISTS))
        {
            return FD_CHASSIS_TYPE_OTHER;
        }
        FILE *file;
        char buff[4] = {0};
        file = fopen (file_path, "r");
        fread (buff, 1, sizeof(buff), file);
        fclose (file);
        chassis_type = atoi (buff);
    }
    g_debug ("%s, chassis_type = %d", __func__, chassis_type);
    if (chassis_type == 8)
    {
        type = FD_CHASSIS_TYPE_PROTABLE;
    }
    else if (chassis_type == 9)
    {
        type = FD_CHASSIS_TYPE_LAPTOP;
    }
    else if (chassis_type == 10)
    {
        type = FD_CHASSIS_TYPE_NOTEBOOK;
    }
    else if (chassis_type == 11)
    {
        type = FD_CHASSIS_TYPE_HAND_HELD;
    }
    else if (chassis_type == 13)
    {
        type = FD_CHASSIS_TYPE_ALL_IN_ONE;
    }
    else if (chassis_type == 14)
    {
        type = FD_CHASSIS_TYPE_SUB_NOTEBOOK;
    }
    else if (chassis_type == 30)
    {
        type = FD_CHASSIS_TYPE_TABLET;
    }
    else
    {
        type = FD_CHASSIS_TYPE_OTHER;
    }
    return type;
}