#ifndef FD_RR_PRIVATE_H
#define FD_RR_PRIVATE_H

#ifdef GDK_WINDOWING_WAYLAND
#include <gdk/gdkwayland.h>
#else
enum wl_output_transform {
  WL_OUTPUT_TRANSFORM_NORMAL,
  WL_OUTPUT_TRANSFORM_90,
  WL_OUTPUT_TRANSFORM_180,
  WL_OUTPUT_TRANSFORM_270,
  WL_OUTPUT_TRANSFORM_FLIPPED,
  WL_OUTPUT_TRANSFORM_FLIPPED_90,
  WL_OUTPUT_TRANSFORM_FLIPPED_180,
  WL_OUTPUT_TRANSFORM_FLIPPED_270
};
#endif

#include "meta-xrandr-shared.h"
#include "meta-dbus-xrandr.h"

typedef struct ScreenInfo ScreenInfo;

struct ScreenInfo
{
    int			min_width;
    int			max_width;
    int			min_height;
    int			max_height;

    guint               serial;
    
    FdRROutput **	outputs;
    FdRRCrtc **	crtcs;
    FdRRMode **	modes;
    
    FdRRScreen *	screen;

    FdRRMode **	clone_modes;

    FdRROutput *     primary;
};

struct FdRRScreenPrivate
{
    GdkScreen *			gdk_screen;
    GdkWindow *			gdk_root;
    ScreenInfo *		info;

    int                         init_name_watch_id;
    MetaDBusDisplayConfig      *proxy;
};

#define UNDEFINED_GROUP_ID 0
struct FdRRTile {
  guint group_id;
  guint flags;
  guint max_horiz_tiles;
  guint max_vert_tiles;
  guint loc_horiz;
  guint loc_vert;
  guint width;
  guint height;
};

typedef struct FdRRTile FdRRTile;

struct _FdRROutputInfoPrivate
{
    char *		name;

    gboolean		on;
    int			width;
    int			height;
    int			rate;
    int			x;
    int			y;
    FdRRRotation	rotation;
    FdRRRotation	available_rotations;

    gboolean		connected;
    char *		vendor;
    char *		product;
    char *		serial;
    double		aspect;
    int			pref_width;
    int			pref_height;
    char *		display_name;
    char *		connector_type;
    gboolean            primary;
    gboolean            underscanning;

    gboolean            is_tiled;
    FdRRTile         tile;

    int                 total_tiled_width;
    int                 total_tiled_height;
    /* ptr back to info */
    FdRRConfig       *config;
};

struct _FdRRConfigPrivate
{
  gboolean clone;
  FdRRScreen *screen;
  FdRROutputInfo **outputs;
};

gboolean _fd_rr_output_connector_type_is_builtin_display (const char *connector_type);

gboolean _fd_rr_screen_apply_configuration (FdRRScreen  *screen,
					       gboolean        persistent,
					       GVariant       *crtcs,
					       GVariant       *outputs,
					       GError        **error);

const char * _fd_rr_output_get_connector_type  (FdRROutput         *output);
gboolean    _fd_rr_output_get_tile_info      (FdRROutput         *output,
						FdRRTile *tile);
gboolean    _fd_rr_output_get_tiled_display_size (FdRROutput *output,
						     int *tile_w, int *tile_h,
						     int *width, int *height);

#endif
