#include "fd-user.h"
#include <grp.h>

#define NOPWD_LOGIN_GROUP "nopasswdlogin"

static gboolean
user_in_nopasswdlogin_group (const char *name, const gchar *group_name) 
{
    struct group *data;
    int i;
    gboolean has = FALSE;

    while (0 != (data = getgrent())) 
    { 
        if (0 == strcmp (data->gr_name, group_name)) 
        {
            i = 0;
            while (data->gr_mem[i]) 
            {
                if (0 == strcmp (name, data->gr_mem[i++])) 
                {
                    has = TRUE;
                    break;
                }
            }
            break;
        }
    }
    endgrent ();
    return has;
}

static void 
add_to_group (const gchar *username, const gchar *group_name)
{
    gchar *command_groupadd;
    gchar *command_gpasswd;

    command_groupadd = g_strdup_printf ("pkexec /usr/sbin/groupadd -f %s", group_name);
    system (command_groupadd);

    command_gpasswd = g_strdup_printf ("pkexec /usr/bin/gpasswd -a %s %s", username, group_name);
    system (command_gpasswd);

    g_free (command_groupadd);
    g_free (command_gpasswd);
}

static void 
remove_from_group (const gchar *username, const gchar *group_name)
{
    gchar *command;
    command = g_strdup_printf ("pkexec /usr/bin/gpasswd -d %s %s", username, group_name);
    system (command);
    g_free (command);
}

gboolean
fd_user_set_nopwd_login (const gchar *username, gboolean enabled)
{
    if (enabled)
        add_to_group (username, NOPWD_LOGIN_GROUP);
    else 
        remove_from_group (username, NOPWD_LOGIN_GROUP);
    return TRUE;
}

gboolean
fd_user_get_nopwd_login (const gchar *username)
{
    return user_in_nopasswdlogin_group (username, NOPWD_LOGIN_GROUP);
}

gboolean 
fd_user_can_support_nopwd_login (const gchar *username)
{
    return TRUE;
}

static void
users_loaded (ActUser       *user,
              GParamSpec    *pspec,
              gpointer       user_data)
{
    GMainLoop *loop = (GMainLoop *)user_data;
    gboolean is_loaded;
    is_loaded = act_user_is_loaded (user);
    g_debug ("User: %s : is_loaded = %d", act_user_get_user_name (user), is_loaded);

    g_main_loop_quit (loop);
}

static ActUser *
fd_get_loaded_act_user (const gchar *username)
{
    ActUserManager *manager = NULL;
    gboolean is_loaded;
    GMainLoop *loop = NULL;
    ActUser       *user;

    manager = act_user_manager_get_default ();
    user = act_user_manager_get_user (manager, username);

    if (!act_user_is_loaded (user))
    {
        loop = g_main_loop_new (NULL, FALSE);
        g_signal_connect (user, 
                          "notify::is-loaded", 
                          G_CALLBACK (users_loaded), 
                          loop);

        g_main_loop_run (loop);
        g_main_loop_unref (loop); 
    }
    return user;
}

//自动登陆
gboolean
fd_user_set_automatic_login (const gchar *username, gboolean enabled)
{
    ActUser *user;
    user = fd_get_loaded_act_user (username);
    if (!user)
        return FALSE;
    if (!act_user_is_loaded (user))
    {
        g_warning ("Users is not loaded in accountsservice.");
        return FALSE;
    }
    act_user_set_automatic_login (user, enabled);
    return TRUE;
}

gboolean
fd_user_get_automatic_login (const gchar *username)
{
    ActUser *user;
    user = fd_get_loaded_act_user (username);
    if (!user)
        return FALSE;
    if (!act_user_is_loaded (user))
    {
        g_warning ("Users is not loaded in accountsservice.");
        return FALSE;
    }
    return act_user_get_automatic_login (user);
}

gboolean 
fd_user_can_support_automatic_login (const gchar *username)
{
    return TRUE;
}

const gchar *
fd_user_get_display_name_by_username (const gchar *username)
{
    ActUser *user;
    user = fd_get_loaded_act_user (username);
    if (!user)
        return username;
    if (!act_user_is_loaded (user))
    {
        g_warning ("Users is not loaded in accountsservice.");
        return username;
    }
    return act_user_get_real_name (user);
}

const gchar *
fd_user_get_display_name_by_user (ActUser *user)
{
    g_return_val_if_fail (ACT_IS_USER (user), NULL);

    const gchar *user_name = act_user_get_real_name (user);
    if (!user_name)
        user_name = act_user_get_user_name (user);
    return user_name;
}
